﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Hornet
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void cmdSend_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", ""); 

            try
            {
                // Log in
                m_Toolbox.Session.Login();
                rtbCommunication.AppendText("Logged into " + txtIP.Text 
                    + ". Uploading files\n");

                // Push SMIL to play it full screen
                string urlTarget = "file:///user-data/index.smil";
                System.IO.MemoryStream ms = new System.IO.MemoryStream(
                    System.Text.Encoding.UTF8.GetBytes(rtbSmil.Text));
                m_Toolbox.PushObject(urlTarget, ms, "application/smil");
                rtbCommunication.AppendText("Uploaded " + urlTarget + "\n");

                // Play!
                m_Toolbox.MediaControl.Play("display:0", urlTarget, "");
                rtbCommunication.AppendText("Play started\n");

                // Log out
                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                tabActions.SelectTab(tpgLogs);
            }

            Cursor.Current = Cursors.Default;
        }

        private void DoSave()
        {
            SaveFileDialog sfd = new SaveFileDialog();

            sfd.Filter = "SMIL/I-SMIL script|*.smi;*.smil|All files|*.*";
            if (DialogResult.OK == sfd.ShowDialog())
            {
                using (System.IO.Stream fs = sfd.OpenFile())
                    if (null != fs)
                        using (System.IO.StreamWriter sw 
                            = new System.IO.StreamWriter(fs, Encoding.UTF8))

                            sw.Write(rtbSmil.Text);

                rtbSmil.Modified = false;
            }
        }

        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            DoSave();
        }

        private void rtbCommunication_TextChanged(object sender, EventArgs e)
        {
            rtbCommunication.ScrollToCaret();
        }

        private void openToolStripButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();

            ofd.Filter = "SMIL/I-SMIL script|*.smi;*.smil|All files|*.*";
            ofd.CheckFileExists = true;

            if (DialogResult.OK == ofd.ShowDialog())
            {
                try
                {
                    System.IO.StreamReader sr = new System.IO.StreamReader(ofd.FileName);
                    rtbSmil.Text = sr.ReadToEnd();
                    rtbSmil.Modified = false;
                }
                catch (Exception)
                {
                    MessageBox.Show("Loading file '" + ofd.FileName + " failed",
                        Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private string smilEmpty = "<smil>\n  <head />\n  <body>\n"
            + "    <!-- Edit this I-SMIL script, or load a sample to test -->\n" 
            + "  </body>\n" + "</smil>";

        private void newToolStripButton_Click(object sender, EventArgs e)
        {
            rtbSmil.Text = smilEmpty;
            rtbSmil.Modified = false;
        }

        private void rtbSmil_ModifiedChanged(object sender, EventArgs e)
        {
            if (rtbSmil.Modified)
                Form1.ActiveForm.Text = Application.ProductName + " *";
            else
                Form1.ActiveForm.Text = Application.ProductName;
        }

        private void GetInventory()
        {
            Cursor.Current = Cursors.WaitCursor;

            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", "");

            try
            {
                m_Toolbox.Session.Login();
                rtbCommunication.AppendText("Logged into " + txtIP.Text
                    + ". Checking inventory\n");

                // Enumerate all objects on player               
                Adapi.ObjectFilter[] Continuation = null;

                Adapi.DeviceReport report = m_Toolbox.Contents.FindObject(
                    Continuation, 0, out Continuation);

                m_Toolbox.UpdateInventory(lvInventory, report);

                // Log out
                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                tabActions.SelectTab(tpgLogs);
            }

            lvInventory.Sorting = SortOrder.Ascending;
            lvInventory.Sort();

            Cursor.Current = Cursors.Default;
        }

        private void cmdPing_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", "");

            bool bPingSuccess = false;
            try
            {
                m_Toolbox.Session.Login();
                rtbCommunication.AppendText("Pinging " + txtIP.Text + "\n");
                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");

                bPingSuccess = true;
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                tabActions.SelectTab(tpgLogs);
            }

            if (bPingSuccess)
            {
                tlsCommands.Enabled = true;
                tlsSamples.Enabled = true;
                tabActions.Enabled = true;
            }

            Cursor.Current = Cursors.Default;
        }

        private void cmdCheckInventory_Click(object sender, EventArgs e)
        {
            tabActions.SelectTab(tpgInventory);
            GetInventory();
        }

        private int UploadFiles(List<string> FullPath, List<string> PlayerUrls)
        {
            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", "");

            int cUploaded = 0;

            try
            {
                m_Toolbox.Session.Login();

                rtbCommunication.AppendText("Sending "
                    + FullPath.Count + " files to " + txtIP.Text + "\n");

                List<string>.Enumerator iPath = FullPath.GetEnumerator(), 
                    iUrl = PlayerUrls.GetEnumerator();

                while( iPath.MoveNext() && iUrl.MoveNext() )
                {
                    string fn = iPath.Current ;
                    string url = Uri.EscapeUriString( iUrl.Current ) ;

                    System.IO.Stream fs = new System.IO.FileStream(fn,
                        System.IO.FileMode.Open, System.IO.FileAccess.Read);

                    if (null!=fs)
                    {
                        rtbCommunication.AppendText("Sending "
                            + fn + " as " + url + "\n");

                        string result = m_Toolbox.PushObject(url, fs, "");
                        if (result.StartsWith("ERROR"))
                        {
                            rtbCommunication.AppendText(result + "\n");
                            break ;
                        }

                        fs.Close();
                    }

                    cUploaded++;
                }

                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                tabActions.SelectTab(tpgLogs);
            }

            return cUploaded;
        }

        private int UploadFiles(string[] FilePaths, string BasePath)
        {
            List<string> PlayerUrls = new List<string>();

            // Remove last '\', if any
            if (null != BasePath && BasePath.EndsWith("\\")) 
                BasePath = BasePath.Substring(0, BasePath.Length-1);

            foreach (string fn in FilePaths)
            {
                if (null == BasePath || BasePath.Length == 0 ||
                    !fn.StartsWith(BasePath))

                    PlayerUrls.Add("file:///user-data/" + System.IO.Path.GetFileName(fn));

                else

                    PlayerUrls.Add("file:///user-data/"
                        + fn.Substring(BasePath.Length + 1).Replace('\\', '/'));

            }

            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", "");

            int cUploaded = 0;

            try
            {
                m_Toolbox.Session.Login();

                rtbCommunication.AppendText("Sending "
                    + FilePaths.Length.ToString() 
                    + " files to " + txtIP.Text + "\n");

                System.Collections.IEnumerator iPath = FilePaths.GetEnumerator(),
                    iUrl = PlayerUrls.GetEnumerator();

                while (iPath.MoveNext() && iUrl.MoveNext())
                {
                    string fn = (string)iPath.Current;
                    string url = (string)iUrl.Current;

                    System.IO.Stream fs = new System.IO.FileStream(fn,
                        System.IO.FileMode.Open, System.IO.FileAccess.Read);

                    if (null != fs)
                    {
                        rtbCommunication.AppendText("Sending "
                            + fn + " as " + url + "\n");

                        string result = m_Toolbox.PushObject(url, fs, "");
                        if (result.StartsWith("ERROR"))
                        {
                            rtbCommunication.AppendText(result + "\n");
                            break;
                        }

                        fs.Close();
                    }

                    cUploaded++;
                }

                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                tabActions.SelectTab(tpgLogs);
            }

            return cUploaded;
        }

        private void cmdUpload_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Title = "Select files to upload";
            ofd.Multiselect = true;
            ofd.CheckFileExists = true;

            ofd.Filter = "Media files|*.mpg;*.wmv;*.mp4;*.mov;*.jpg;*.png;*.gif;*.mp3|All files|*.*";
            if (DialogResult.OK == ofd.ShowDialog())
            {
                Cursor.Current = Cursors.WaitCursor;

                int cUploaded = UploadFiles(ofd.FileNames, null);

                if (cUploaded < ofd.FileNames.Length)
                {
                    MessageBox.Show("Upload failed after transfering " + cUploaded.ToString()
                        + " files", Application.ProductName, MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }

                Cursor.Current = Cursors.Default;
            }
        }

        private void runSMILToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (1 == lvInventory.SelectedItems.Count)
            {
                ListViewItem lvi = lvInventory.SelectedItems[0];

                string url = lvi.Text;

                PlaySmilUrl(url);
            }
        }

        private void deleteFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", "");

            try
            {
                // Log in
                m_Toolbox.Session.Login();
                tabActions.SelectTab(tpgLogs);
                rtbCommunication.AppendText("Logged into " + txtIP.Text
                    + ". Deleting files\n");

                foreach (ListViewItem lvi in lvInventory.SelectedItems)
                {
                    string url = lvi.Text;

                    rtbCommunication.AppendText("Deleting " + url + "\n");
                    Application.DoEvents();

                    bool bSuccess = false;
                    for( int iTries = 0; !bSuccess && iTries < 3; iTries++)
                        try
                        {
                            m_Toolbox.Contents.RemoveResource(url);
                            bSuccess = true;
                        }
                        catch (Exception ex)
                        {
                        }

                    if(!bSuccess)
                        rtbCommunication.AppendText("ERROR: Failed to delete " + url + "\n");
                }

                // Log out
                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
            }

            Cursor.Current = Cursors.Default;
        }

        private void PlaySmilUrl(string urlSmil)
        {
            Cursor.Current = Cursors.WaitCursor;

            AdApiWrapper m_Toolbox = new AdApiWrapper(
                "http://" + txtIP.Text, "admin", "");

            try
            {
                // Log in
                m_Toolbox.Session.Login();
                rtbCommunication.AppendText("Logged into " + txtIP.Text
                    + ". Playing SMIL " + urlSmil + "\n");

                // Play!
                m_Toolbox.MediaControl.Play("display:0", urlSmil, "");
                rtbCommunication.AppendText("Play started\n");

                // Log out
                m_Toolbox.Session.Logout();
                rtbCommunication.AppendText("DONE\n");
            }
            catch (Exception ex)
            {
                rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                tabActions.SelectTab(tpgLogs);
            }

            Cursor.Current = Cursors.Default;
        }

        private void cmsInventory_Opening(object sender, CancelEventArgs e)
        {
            if (0 == lvInventory.SelectedItems.Count)
            {
                runSMILToolStripMenuItem.Enabled = false;
                deleteFileToolStripMenuItem.Enabled = false;
            }
            else
            {
                deleteFileToolStripMenuItem.Enabled = true;
                runSMILToolStripMenuItem.Enabled = false;

                if (1 == lvInventory.SelectedItems.Count)
                {
                    ListViewItem lvi = lvInventory.SelectedItems[0];

                    string url = lvi.Text;
                    string type = lvi.SubItems[0].Text;

                    // TODO: hack to infer SMIL type
                    if (url.EndsWith(".smil", StringComparison.CurrentCultureIgnoreCase)
                        || url.EndsWith(".smi", StringComparison.CurrentCultureIgnoreCase))
                    {
                        type = "application/smil";
                    }

                    if ("application/smil" == type)
                        runSMILToolStripMenuItem.Enabled = true;
                }
            }
        }

        private void AlertCannotLoadSample()
        {
            MessageBox.Show("Unable to load sample code. Installation may be corrupt.",
                Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                string CurrentDir = Application.StartupPath;
                if (CurrentDir.EndsWith(@"\bin\Debug"))
                    // Debug mode path
                    CurrentDir = CurrentDir.Substring(0, CurrentDir.Length - @"\bin\Debug".Length);

                string SamplesDir = CurrentDir + @"\samples";

                string[] Topics = System.IO.Directory.GetDirectories(SamplesDir);

                foreach (string TopicDir in Topics)
                {
                    ToolStripDropDownButton tsdd = new ToolStripDropDownButton(
                            System.IO.Path.GetFileName(TopicDir));
                    tsdd.DisplayStyle = ToolStripItemDisplayStyle.Text;
                    tsdd.AutoToolTip = false;

                    tlsSamples.Items.Add(tsdd);

                    string[] Codes = System.IO.Directory.GetDirectories(TopicDir);

                    foreach (string CodeDir in Codes)
                    {
                        ToolStripItem tsi = tsdd.DropDownItems.Add(
                            System.IO.Path.GetFileName(CodeDir));

                        tsi.DisplayStyle = ToolStripItemDisplayStyle.Text;
                        tsi.AutoToolTip = false;
                        tsi.Tag = CodeDir;
                        tsi.Click += new EventHandler(tsiSamples_Click);
                    }
                }
            }
            catch (Exception)
            {
                AlertCannotLoadSample();
            }
        }

        void tsiSamples_Click(object sender, EventArgs e)
        {
            ToolStripItem tsi = (ToolStripItem)sender;
            string SampleDir = tsi.Tag.ToString();

            string[] Files = System.IO.Directory.GetFiles(SampleDir);

            bool bLoadSuccess = true;

            if (Files.Length > 1)
            {
                DialogResult dr = MessageBox.Show("Load total " + Files.Length.ToString()
                    + " sample files to player now?",
                    Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (DialogResult.Yes == dr)
                {
                    tabActions.SelectTab(tpgLogs);
                    bLoadSuccess = (Files.Length == UploadFiles(Files, SampleDir));
                }
                else
                {
                    MessageBox.Show("Sample not loaded", Application.ProductName,
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);

                    bLoadSuccess = false;
                }
            }

            if (bLoadSuccess)
            {
                foreach (string SampleFile in Files)
                {
                    if (SampleFile.EndsWith(@"\index.smil")
                        || SampleFile.EndsWith(@"\index.smi"))
                    {
                        try
                        {
                            System.IO.StreamReader sr = new System.IO.StreamReader(SampleFile);
                            rtbSmil.Text = sr.ReadToEnd();
                            rtbSmil.Modified = false;
                        }
                        catch (Exception ex)
                        {
                            rtbCommunication.AppendText("ERROR: " + ex.Message + "\n");
                            tabActions.SelectTab(tpgLogs);

                            AlertCannotLoadSample();
                            bLoadSuccess = false;
                        }
                    }
                }

                if( bLoadSuccess )
                    MessageBox.Show("Edit the SMIL script and click the\n"
                        + "'" + cmdSend.Text + "' button to run sample code",
                        Application.ProductName, 
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void tabActions_Selected(object sender, TabControlEventArgs e)
        {
            if( 1 == e.TabPageIndex )
                GetInventory();
        }

        private void timUpdateSmil_Tick(object sender, EventArgs e)
        {
            System.Xml.XmlDocument doc = new System.Xml.XmlDocument();

            try
            {
                doc.LoadXml(rtbSmil.Text);
            }
            catch( Exception ex )
            {

            }
        }

        private void rtbSmil_TextChanged(object sender, EventArgs e)
        {
            timUpdateSmil.Interval = 500;
        }
    }

    public class AdApiWrapper
    {
        public Adapi.Session Session;
        public Adapi.MediaControl MediaControl;
        public Adapi.ContentManager Contents;

        private string m_ServerRootUrl;
        private System.Net.CookieContainer m_SessionCookies;

        public AdApiWrapper(string ServerRootUrl, string UserName, string Password)
        {
            m_ServerRootUrl = ServerRootUrl;

            Session = new Adapi.Session();
            MediaControl = new Adapi.MediaControl();
            Contents = new Adapi.ContentManager();

            System.Net.NetworkCredential credential = new
                System.Net.NetworkCredential(UserName, Password);
            Session.Credentials = credential;

            System.Net.CredentialCache cc = new System.Net.CredentialCache();
            cc.Add(new Uri(m_ServerRootUrl), "Digest", credential);
            Session.Credentials = cc;

            m_SessionCookies = new System.Net.CookieContainer();
            Session.CookieContainer = m_SessionCookies;
            MediaControl.CookieContainer = m_SessionCookies;
            Contents.CookieContainer = m_SessionCookies;

            Session.Url = new Uri(new Uri(m_ServerRootUrl),
                new Uri(Session.Url).PathAndQuery).ToString();
            MediaControl.Url = new Uri(new Uri(m_ServerRootUrl),
                new Uri(MediaControl.Url).PathAndQuery).ToString();
            Contents.Url = new Uri(new Uri(m_ServerRootUrl),
                new Uri(Contents.Url).PathAndQuery).ToString();
        }

        public string PushObject(string urlTarget, System.IO.Stream payloadStream,
            string MimeType)
        {
            long cbPayload = payloadStream.Length;
            string contentType = MimeType;

            string puturlIndex;
            string objidIndex = Contents.CreateObject(
                urlTarget, contentType, (ulong)cbPayload, DateTime.Now,
                "", "", Adapi.DigestAlgorithm.NONE, 0, null, out puturlIndex);

            string urlPut = new Uri(new Uri(m_ServerRootUrl), puturlIndex).ToString();

            string sResult = objidIndex;

            bool bSuccess = false;
            for (int iTries = 0; !bSuccess && iTries < 3; iTries++)
            {
                // Reset any errors from previous attempts
                sResult = objidIndex;

                System.Net.HttpWebRequest req = (System.Net.HttpWebRequest)
                    System.Net.WebRequest.Create(urlPut);
                req.Method = "PUT";
                req.ContentType = contentType;
                req.ContentLength = cbPayload;
                req.CookieContainer = m_SessionCookies; // to track session
                req.Timeout = System.Math.Max(100000, 
                    (int)(cbPayload * 8 / 10000 * 1000)); // assume speed around 10kbps

                try
                {
                    System.IO.Stream reqStream = req.GetRequestStream();

                    byte[] buffer = new byte[32 * 1024];
                    int read;

                    payloadStream.Seek(0, System.IO.SeekOrigin.Begin);
                    while (0 < (read = payloadStream.Read(buffer, 0, buffer.Length)))
                    {
                        reqStream.Write(buffer, 0, read);
                        Application.DoEvents();
                    }

                    System.Net.HttpWebResponse resp
                        = (System.Net.HttpWebResponse)req.GetResponse();
                    resp.Close();

                    System.Net.HttpStatusCode result = resp.StatusCode;
                    bool bPutOk = (System.Net.HttpStatusCode.OK == result);
                    if (!bPutOk)
                        sResult = "ERROR: " + resp.StatusCode;
                    else
                        bSuccess = true;

                    reqStream.Close();
                }
                catch (Exception ex)
                {
                    sResult = "ERROR: " + ex.Message;
                }

            } // for (int iTries = 0; iTries < 3; iTries++)

            return sResult;
        }

        public void UpdateInventory(ListView lv, Adapi.DeviceReport report)
        {
            if (null == report.player[0])
                return;

            lv.Items.Clear();

            foreach (Adapi.Object obj in report.player[0].contentDirectory.objectInfo)
            {
                ListViewItem lvi = new ListViewItem( obj.resourceURI ) ;
                lvi.SubItems.Add(obj.contentType) ;
                if (null != obj.lastModified) lvi.SubItems.Add(obj.lastModified.ToString());
                lvi.SubItems.Add(obj.transferLength.ToString());
                lv.Items.Add(lvi);
            }

        }

        public bool DeleteUrl(string url)
        {
            Contents.RemoveResource(url);
            return true;
        }
    }
}
